package com.biganto.visual.roompark.presentation.screen.article

import android.text.Html
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import androidx.core.os.bundleOf
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.google.android.material.floatingactionbutton.FloatingActionButton
import com.google.android.material.textview.MaterialTextView
import com.squareup.picasso.Picasso
import jp.wasabeef.picasso.transformations.BlurTransformation
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

private const val ARTICLE_ID = "DIRECT_ARTICLE_ID_KEY"

class ArticleScreenController  :
    BigantoBaseController<ArticleScreenViewState
        , ArticleScreen
        , ArticleScreenPresenter>
    , ArticleScreen {


    constructor()

    constructor(id:Int):super(bundleOf(ARTICLE_ID to id))

    override fun injectDependencies() {
        getComponent()
    }

    @Inject
    override lateinit var injectedPresenter: ArticleScreenPresenter

    @BindView(R.id.articleTitle)
    lateinit var title:MaterialTextView

    @BindView(R.id.articleHeaderBlock)
    lateinit var headerBlock: ViewGroup

    @BindView(R.id.articleContent)
    lateinit var contentView: MaterialTextView

    val blurPreview:ImageView by lazy {
       headerBlock.findViewById<ImageView>(R.id.articlePreviewBlurred)
    }

    val articlePreview:ImageView by lazy {
        headerBlock.findViewById<ImageView>(R.id.articlePreview)
    }

    val articleDate:MaterialTextView by lazy {
        headerBlock.findViewById<MaterialTextView>(R.id.articleDate)
    }

    val closeButton:FloatingActionButton by lazy {
        headerBlock.findViewById<FloatingActionButton>(R.id.articleCloseButton)
    }



    private fun setToolbar(){
        toolBar.hideAll()
        toolBar.appBar.setExpanded(false,false)
//        toolBar.collapsingToolbarLayout.title = "ИЗБРАННОЕ"
//        toolBar.appBar.liftOnScrollTargetViewId = R.id.Article_cards_recycler_view
        toolBar.appBar.setLiftable(false)
        toolBar.appBarScrollable(false)
//        ArticleRecyclerView.isNestedScrollingEnabled = false
    }

    private fun bindRecycler() {

    }

    override fun onViewBound(v: View) {
        setToolbar()
        bindRecycler()
    }

    override fun render(viewState: ArticleScreenViewState) {
        super.render(viewState)
        Timber.d("Render state $viewState")
        when(viewState){
            is ArticleScreenViewState.Idle -> render(viewState)
            is ArticleScreenViewState.ArticleLoaded -> render(viewState)
        }
    }

    private fun render(viewState: ArticleScreenViewState.Idle){

    }


    private fun render(viewState: ArticleScreenViewState.ArticleLoaded) {

        title.text = viewState.item.title
        contentView.text = Html.fromHtml(viewState.item.htmlBody)
        articleDate.text = viewState.item.published.toString()

        Picasso.get()
            .load(viewState.item.previewUrl)
            .transform(BlurTransformation(activity,40,4))
            .into(blurPreview)

        Picasso.get()
            .load(viewState.item.previewUrl)
            .into(articlePreview)
    }

    private fun getComponent() = DaggerArticleScreenComponent.factory()
        .create(RoomParkApplication.component,activity as RoomParkMainActivity)
        .inject(this)

    override fun getLayoutId(): Int = R.layout.feed_read_screen


}