package com.biganto.visual.roompark.presentation.screen.feeds

import android.annotation.SuppressLint
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.domain.model.FeedModel
import com.biganto.visual.roompark.presentation.screen.feed_list.ArticlesScreenController
import com.biganto.visual.roompark.presentation.screen.feeds.utils.AlbumsPreviewAdapter
import com.biganto.visual.roompark.presentation.screen.feeds.utils.ArticlesPreviewAdapter
import com.biganto.visual.roompark.presentation.screen.feeds.utils.CamsListAdapter
import com.bluelinelabs.conductor.RouterTransaction
import com.bluelinelabs.conductor.changehandler.FadeChangeHandler
import com.google.android.material.button.MaterialButton
import com.google.android.material.tabs.TabLayout
import com.google.android.material.textview.MaterialTextView
import com.jakewharton.rxbinding3.material.selections
import com.jakewharton.rxbinding3.view.clicks
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import timber.log.Timber
import java.util.concurrent.TimeUnit
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

class FeedsScreenController  :
    BigantoBaseController<FeedsScreenViewState
        , FeedsScreen
        , FeedsScreenPresenter>()
    , FeedsScreen {

    override fun feedsTabSelected(): Observable<Int>
            = feedsTabs.selections()
        .debounce (200L, TimeUnit.MILLISECONDS)
        .map { it.position }
        .map { storedFeedsList[it].feedId }
        .observeOn(AndroidSchedulers.mainThread())
        .doOnNext { feedsBlockView.findViewById<MaterialButton>(R.id.to_feed_articles)
            .text = "ВСЕ ${allFeedName(it)}" }



    override fun injectDependencies() {
        getComponent()
    }

    @BindView(R.id.feedsTabs)
    lateinit var feedsTabs:TabLayout

    @BindView(R.id.feedsRecyclerView)
    lateinit var feedsRecyclerView:RecyclerView

    @BindView(R.id.dev_progress_recycler_view)
    lateinit var devProgressRecyclerView:RecyclerView

    @BindView(R.id.cams_recycler_view)
    lateinit var camsRecyclerView:RecyclerView

    @BindView(R.id.feedsBlock)
    lateinit var feedsBlockView:ViewGroup


    private fun bindRecycler(){
        feedsRecyclerView.isNestedScrollingEnabled = true
        feedsRecyclerView.layoutManager =
            LinearLayoutManager(activity, RecyclerView.VERTICAL, false)
        feedsRecyclerView.adapter = ArticlesPreviewAdapter()
        feedsRecyclerView.itemAnimator = null


        devProgressRecyclerView.isNestedScrollingEnabled = true
        devProgressRecyclerView.layoutManager =
            LinearLayoutManager(activity, RecyclerView.HORIZONTAL, false)
        devProgressRecyclerView.adapter = AlbumsPreviewAdapter()
        devProgressRecyclerView.itemAnimator = null

        camsRecyclerView.isNestedScrollingEnabled = true
        camsRecyclerView.layoutManager =
            LinearLayoutManager(activity, RecyclerView.VERTICAL, false)
        camsRecyclerView.adapter = CamsListAdapter()
        camsRecyclerView.itemAnimator = null
    }

    private var storedFeedsList:List<FeedModel> = arrayListOf()

    @Inject
    override lateinit var injectedPresenter: FeedsScreenPresenter

    override fun onViewBound(v: View) {
        toolBar.hideAll()
        bindRecycler()
        feedsBlockView.findViewById<MaterialTextView>(R.id.to_feed_articles).clicks()
            .map{ feedsTabs.getTabAt(feedsTabs.selectedTabPosition)}
            .map { it.tag as? Int }
            .subscribe {
                it?.let {
                    router.pushController(
                        RouterTransaction.with(ArticlesScreenController(it))
                            .popChangeHandler(FadeChangeHandler())
                            .pushChangeHandler(FadeChangeHandler())
                    )
                }
            }
    }

    override fun render(viewState: FeedsScreenViewState) {
        super.render(viewState)
        Timber.d("Render state $viewState")
        when(viewState){
            is FeedsScreenViewState.Idle -> render(viewState)
            is FeedsScreenViewState.FeedsPages -> render(viewState)
            is FeedsScreenViewState.AlbumsPages -> render(viewState)
            is FeedsScreenViewState.CamsList -> render(viewState)
            is FeedsScreenViewState.GetFeedArticlesPreview -> render(viewState)
            is FeedsScreenViewState.RestoreView -> render(viewState)
            is FeedsScreenViewState.SomeError -> render(viewState)
        }
    }

    private fun render(viewState: FeedsScreenViewState.Idle){

    }

    private fun render(viewState: FeedsScreenViewState.SomeError) =
        showError(viewState.exception)

    private fun allFeedName(feedId:Int) = if (feedId==1) "НОВОСТИ" else "БЛОГИ"

    @SuppressLint("SetTextI18n")
    private fun render(viewState: FeedsScreenViewState.FeedsPages){
        feedsTabs.removeAllTabs()

        viewState.items.forEach {feed ->
            val tab = feedsTabs.newTab()
                .setCustomView(R.layout.feeds_tab_view)
                .setTag(feed.feedId)
            tab.customView
                ?.let {
                    it.findViewById<TextView>(R.id.tab_title)?.text = feed.title

                    it.findViewById<TextView>(R.id.tab_divider)?.visibility =
                        if (viewState.items.indexOf(feed) == viewState.items.size - 1) View.GONE
                        else View.VISIBLE
                }
            feedsTabs.addTab(tab)
        }
        storedFeedsList = viewState.items
    }

    private fun render(viewState: FeedsScreenViewState.AlbumsPages){
        (devProgressRecyclerView.adapter as AlbumsPreviewAdapter).setItems(viewState.items)
    }

    private fun render(viewState: FeedsScreenViewState.GetFeedArticlesPreview){
        (feedsRecyclerView.adapter as ArticlesPreviewAdapter).setItems(viewState.items)
    }

    private fun render(viewState: FeedsScreenViewState.CamsList){
        (camsRecyclerView.adapter as CamsListAdapter).setItems(viewState.items)
    }


    private fun render(viewState: FeedsScreenViewState.RestoreView){
        (camsRecyclerView.adapter as CamsListAdapter).setItems(viewState.restore.cams)
        (feedsRecyclerView.adapter as ArticlesPreviewAdapter).setItems(viewState.restore.articles)
        (devProgressRecyclerView.adapter as AlbumsPreviewAdapter).setItems(viewState.restore.albums)
        feedsTabs.removeAllTabs()

        viewState.restore.feeds.forEach {feed ->
            val tab = feedsTabs.newTab()
                .setCustomView(R.layout.feeds_tab_view)
                .setTag(feed.feedId)
            tab.customView
                ?.let {
                    it.findViewById<TextView>(R.id.tab_title)?.text = feed.title

                    it.findViewById<TextView>(R.id.tab_divider)?.visibility =
                        if (viewState.restore.feeds.indexOf(feed) == viewState.restore.feeds.size - 1) View.GONE
                        else View.VISIBLE
                }
            feedsTabs.addTab(tab)
        }
        storedFeedsList = viewState.restore.feeds
    }

    private fun getComponent() = DaggerFeedsScreenComponent.factory()
        .create(RoomParkApplication.component,activity as RoomParkMainActivity)
        .inject(this)

    override fun getLayoutId(): Int = R.layout.feeds_screen


    override fun handleBack(): Boolean {
        Timber.d("handle back in class ")
        router.popController(this)
        return super.handleBack()
    }

}